!
! inttest --- test the numerically integration routines
! Christian Buth
! 31 Aug 2006
!

      program inttest
      use fortrantl
      implicit none

      integer i, ngrid
      real(dblprec) xh
      real(dblprec), allocatable :: xc(:), yc(:)

      write (*,*) 'The integrated function is x**2 + x * sin(100*x) from 0 to 1'
      write (*,*)
      write (*,*) 'A grid of values are integrated.  The interval length is adjusted'
      write (*,*) 'according to the specific Newton-Cotes formula, i.e., the'
      write (*,*) 'higher order formulas use a larger interval because they would'
      write (*,*) 'need intermediate points otherwise.'
      write (*,*)
      write (*,*) 'ngrid, midpoint, trapezoid, simpsons, and 3/8-rule are'
      ngrid=13
      do i=1, 12
        call quadratfuncgrid(ngrid)
        ngrid = ngrid * 2 + 1
      end do


      write (*,*)
      write (*,*) 'A grid of values are integrated.  The interval length is not adjusted!'
      write (*,*) 'Intermediate points are calculated as needed.'
      write (*,*)
      write (*,*) 'ngrid, midpoint, trapezoid, simpsons, and 3/8-rule are'
      ngrid=13
      do i=1, 12
        call quadratfuncinterval(ngrid)
        ngrid = ngrid * 2 + 1
      end do

      deallocate(xc, yc)


      contains

!
!    samplefunc --- represent the function by a grid of numerical values
!

      subroutine samplefunc(ngrid)
      use quadrature
      implicit none

      integer ngrid, i
      real(dblprec) xmin, xmax, x

      if (allocated(xc)) then
        deallocate(xc, yc)
      end if
      allocate(xc(ngrid), yc(ngrid))

      xmin = 0.0D0
      xmax = 1.0D0
      xh = (xmax - xmin) / real(ngrid-1, dblprec)

      do i=1, ngrid
        x = real(i-1, dblprec) * xh
        xc(i) = x
        yc(i) = x**2 + x * sin(100*x)
        if (ngrid == 451) then
          write (100,*) xc(i), yc(i)
        end if
      end do
      end subroutine samplefunc


!
!    quadratfuncgrid --- quadrature of a function on a grid
!

      subroutine quadratfuncgrid(ngrid)
      use quadrature
      implicit none

      integer ngrid, ng
      real(dblprec) intmid, inttrap, intsimp, int38

      ng = ngrid - mod(ngrid, 3) + 1
      if (.not. btest(ng, 0)) then
        ng = ng + 3
      end if
      call samplefunc(ng)

      intmid  = midpointrule    (ng, xh, yc)
      inttrap = trapezoidrule   (ng, xh, yc)
      intsimp = simpsonsrule    (ng, xh, yc)
      int38   = threeeighthsrule(ng, xh, yc)

      write (*,'(I8,4F20.16)') ng, intmid, inttrap, intsimp, int38
      end subroutine quadratfuncgrid


!
!    quadratfuncinterval --- quadrature of a function on an interval
!

      subroutine quadratfuncinterval(ngrid)
      use quadrature
      implicit none

      integer ngrid
      real(dblprec) intmid, inttrap, intsimp, int38

      call samplefunc(ngrid / 4)
      intmid  = midpointrule    (ngrid / 4, xh, yc)
      inttrap = trapezoidrule   (ngrid / 4, xh, yc)
      call samplefunc(ngrid / 2 + mod(ngrid / 2, 2) + 1)
      intsimp = simpsonsrule    (ngrid / 2 + mod(ngrid / 2, 2) + 1, xh, yc)
      call samplefunc(ngrid - mod(ngrid, 3) + 1)
      int38   = threeeighthsrule(ngrid - mod(ngrid, 3) + 1, xh, yc)

      write (*,'(I8,4F20.16)') ngrid, intmid, inttrap, intsimp, int38
      end subroutine quadratfuncinterval
      end program inttest
