!
! lanczosdiagfloquet --- diagonalize the Floquet matrix using the
!       band Lanczos algorithm
! Christian Buth
! 19 May 2006
!


      subroutine lanczosdiagfloquet(ceig, polest, nmat, nmain, niter, istart, admixture)
      use floquetmat, lanczosdiagfloquetdbl => lanczosdiagfloquet
      use sparse
      use lanczos
      implicit none

      integer, intent(in) :: nmat, nmain, niter, istart(nmain)
      real(dblprec), intent(in) :: admixture
      real(dblprec), intent(out) :: polest(niter)
      complex(dblprec), intent(out) :: ceig(niter)

      integer, parameter :: maxnumeigenvectors = 100
      integer i, neigenv, ncartsv, jnd, iparam(10), icartsv(1),  &
     &          evlist(maxnumeigenvectors)
      real(dblprec) thresh(3), r1, r2
      complex(dblprec) zt
      complex(dblprec), allocatable :: lanczosvectors(:,:,:), &
     &          Tmat(:,:), e2(:), z(:,:), zev(:,:), ev(:,:)
      external matrixvector

      write(*,*) 'Diagonalizing the complex symmetric Hamiltonian ', &
     &           'with dimension =', nmat
      write (*,*) 'Number of band-Lanczos iterations ', niter
      write (*,*) 'Number of Lanczos vectors ', nmain

      ! Error checking
      if (niter > nmat) then
        write (*,*) 'Warning: the number of band-Lanczos iterations is larger ',  &
     &              'than the matrix dimension!'
      end if

      ! Allocate memory
      neigenv = 1               ! For the memory allocation to work
      allocate(lanczosvectors(nmat, nmain, 3), Tmat(nmain+1, niter), &
     &          e2(niter), z(2*nmain, niter), zev(2*nmain, niter),   &
     &          ev(nmat,neigenv))

      ! Read zHmat from disc
      sparsematfile%readfile = .true.
      sparsematfile%fmtxcom  = 393
      sparsematfile%fmtxdia  = 394
      sparsematfile%fmtxoff  = 395
      call openmatrixfile(sparsematfile)

      lanczosvectors = 0
      ! Generate random number vector start vectors
      do i=1, nmat
        call random_number(r1)
        call random_number(r2)
        lanczosvectors(i, 1, 2) = cmplx(r1, r2, dblprec)
      end do

      ! Normalize the start vector, dot_product is a Hermitian product
      zt = dot_product(conjg(lanczosvectors(:, 1, 2)), lanczosvectors(:, 1, 2))
      zt = sqrt(zt) / admixture
      lanczosvectors(:, 1, 2) = lanczosvectors(:, 1, 2) / zt

      ! Generate a cartesian start vector with random admixture
      lanczosvectors(istart(1), 1, 2) = lanczosvectors(istart(1), 1, 2) + (1.0D0, 0.0D0)
      zt = dot_product(conjg(lanczosvectors(:, 1, 2)), lanczosvectors(:, 1, 2))
      lanczosvectors(:, 1, 2) = lanczosvectors(:, 1, 2) / sqrt(zt)

      ! Set parameters for complex symmetric band-Lanczos
      neigenv     = 0          ! Normal Lanczos run
      ncartsv     = 0
      iparam(1)   = 1
      iparam(2)   = niter
      iparam(3:6) = 0
      call zblanc(nmat, nmain, niter, neigenv, ncartsv, iparam, Tmat,  &
     &          lanczosvectors, z, ev, icartsv, matrixvector)

      ! Set parameters for zbnddia
      thresh(1) = 0.07          ! Minimal pole strength for output
      thresh(2) = 1
      thresh(3) = 0
      neigenv   = 0
      evlist(1) = 0
      iparam(1) = 0
      ceig(:)   = (0.0D0, 0.0D0)
      call zbnddia(niter, nmain, Tmat, ceig, e2, zev, thresh,  &
     &          neigenv, evlist, iparam, jnd, polest)

      call closematrixfile(sparsematfile)
      deallocate(lanczosvectors, Tmat, e2, z, zev, ev)
      end subroutine lanczosdiagfloquet
