!
! eigenfxn --- evaluate the eigenfunctions on a grid
! Kevin Christ, Mark Baertschy, Christian Buth
! 24 Aug 2006
!


      subroutine eigenfxn(egvec, ngrd, rf, xh, nbasis, nstate, nl, xnd, egfxn, egder)
      use fortrantl
      use FiniteElements
      implicit none

      integer ngrd, nbasis, nstate, nl
      integer j, k
      real(dblprec) egvec(nbasis, nstate, nl), egfxn(ngrd, nstate, nl),  &
     &          egder(ngrd, nstate, nl), rf
      real(dblprec), allocatable ::  x(:), rr(:)
      real(dblprec) xf, xh
      type(FEBnodes) xnd
      integer it0, it1, ict

Print*,'Evaluating eigenfunctions'

Print*,'The number of grid points for numerical function evaluation is', ngrd

!! Evaluate eigenfunctions on a grid rr
!! ngrd is the number of grid points
!! rf is the size of the grid
!! xh is the spacing of the even grid
!! nbasis is the number of basis functions
!! nstate is the number of eigenstates per l
!! nl is the number of l quantum numbers considered

CALL system_clock( it0, ict )

ALLOCATE( x(ngrd), rr(ngrd) )

!! Create square root mesh

   xf = dsqrt(rf)
   xh = xf / dble(ngrd)
   do j=1, ngrd
     x(j) = xh * dble(j)
     rr(j) = x(j) * x(j)
   end do


egfxn(:,:,:) = 0.0D0

!! columns of egfxn will contain the first few eigenfunctions
!! the input eigenvectors are the columns of egvec
!! fxneval evaluates, on a grid, a function given by finite
!! element coefficients

      do k = 1, nl
        do j = 1, nstate
          call fxneval1(egfxn(1,j,k), egder(1,j,k), rr, ngrd, egvec(1,j,k), xnd)
        end do
      end do

      call system_clock(it1)
      Print*,'Time is', (it1-it0)*1.0D0/ict
      end subroutine eigenfxn
