!
! getdipole --- read one-electron integrals and extract given
!       dipole matrix elements
! Christian Buth
! 15 Nov 2006
!


      program getdipole
      use floquetmat
      use anglib
      implicit none

      integer nstate, nvlnc, nl, li, init, lmin, lmax, i, nmatel,  &
     &          magnetic, l,  &
     &          idx1(50), idx2(50), ang1(50), ang2(50)
      real(dblprec) Eground,  &             ! Energy of initial valence level
     &              angpart
      real(dblprec), allocatable :: eval(:,:)
      complex(dblprec), allocatable ::              &
     &          SESWatom(:,:,:),                    &
     &          SESdipatm(:,:,:,:),                 &
     &          SESpotatm(:,:,:),                   &
     &          SESfTfatm(:,:,:)

      namelist/radialdipole/ lmin, lmax, nstate, nmatel, idx1, idx2, ang1, ang2

      ! Not needed
      li = 1
      init = 1

      ! Default parameters
      nstate = 20
      nl = 3
      lmin = 0 ; lmax = 2
      nmatel = 0
      read(*,nml=radialdipole)
      nl = lmax - lmin + 1

      allocate(eval(nstate,lmin:lmax),                       &
     &          SESWatom(nstate,nstate,lmin:lmax),             &
     &          SESdipatm(nstate,lmin:lmax,nstate,lmin:lmax),  &
     &          SESpotatm(nstate,nstate,lmin:lmax),            &
     &          SESfTfatm(nstate,nstate,lmin:lmax))
      call readatom(nl, lmin, lmax, nstate, nvlnc, eval, SESdipatm,  &
     &              SESfTfatm, SESpotatm, SESWatom, li, init, Eground)

      write(*,*) 'Number of the S shell of the valence electron, nvlnc =', nvlnc
      write(*,*)
      write(*,*) 'Radial part of the matrix elements'

      write (*,*) 'No, j, l1, k, l2, SESdipatm(j,l1,k,l2)'
      do i=1, nmatel
        write (*,*) i, idx1(i), ang1(i), idx2(i), ang2(i),  &
     &          SESdipatm(idx1(i), ang1(i), idx2(i), ang2(i))
      end do
      deallocate(eval, SESWatom, SESdipatm, SESpotatm, SESfTfatm)

      write(*,*)
      write(*,*) 'Angular part of the matrix elements'
      write (*,*) 'No, l1, l2, m, angpart'
      ! Angular part of the matrix elements
      do i=1, nmatel
	l = min(ang1(i), ang2(i))
	do magnetic=-l, l
	  angpart = sqrt(real(2*ang2(i)+1) / real(2*ang1(i)+1))
	  angpart = angpart * cleb(2*ang2(i), 2*magnetic, 2*1, 0, 2*ang1(i), 2*magnetic)
	  angpart = angpart * cleb(2*ang2(i),          0, 2*1, 0, 2*ang1(i),          0)
          write (*,*) i, ang1(i), ang2(i), magnetic, angpart
        end do
	write (*,*)
      end do
      end program getdipole
